{
    "positive": {
        "required": False,
        "type": "string",
        "nullable": False,
    },
    "négative": {
        "required": False,
        "type": "string",
        "nullable": False,
    },
    "erreur": {
        "required": False,
        "type": "string",
        "nullable": False,
    },
}



#!/usr/bin/python3
from cerberus import Validator, schema_registry
import argparse
import json
import os
import pathlib
import re
import sys
import urllib
import yaml
import yamlinclude
import werkzeug.urls

try:
    import importlib.resources as pkg_resources
except:
    import importlib_resources as pkg_resources

from . import schemas

__args__ = None


def traiter_validation_question_yaml(contenu_yaml, schema):
    v = Validator(schema)
    v.allow_unknown = True
    v.validate(contenu_yaml)
    erreurs = v.errors

    v.allow_unknown = False
    v.validate(contenu_yaml, normalize=True)

    avertissements = {k: v.errors[k] for k in v.errors if k not in erreurs}

    return {"erreurs": erreurs, "avertissements": avertissements}


def valider_schema_yaml_infos_question(contenu_fichier):
    schema_registry.add(
        "rétroactions", eval(pkg_resources.read_text(schemas, "rétroactions.py"))
    )
    schema_registry.add(
        "question_base", eval(pkg_resources.read_text(schemas, "question_base.py"))
    )
    schema_registry.add(
        "question_prog",
        {
            **schema_registry.get("question_base"),
            **eval(pkg_resources.read_text(schemas, "question_prog.py")),
        },
    )
    schema_registry.add(
        "question_sys",
        {
            **schema_registry.get("question_base"),
            **eval(pkg_resources.read_text(schemas, "question_sys.py")),
        },
    )

    schema_registry.add(
        "question_vcs",
        {
            **schema_registry.get("question_sys"),
            **eval(pkg_resources.read_text(schemas, "question_vcs.py")),
        },
    )

    schema_registry.add(
        "question_bd",
        {
            **schema_registry.get("question_sys"),
            **eval(pkg_resources.read_text(schemas, "question_bd.py")),
        },
    )

    if "type" in contenu_fichier and contenu_fichier["type"].lower() == "prog":
        schemas_à_valider = "question_prog"
    elif "type" in contenu_fichier and contenu_fichier["type"].lower() == "vcs":
        schemas_à_valider = "question_vcs"
    elif "type" in contenu_fichier and contenu_fichier["type"].lower() == "sys":
        schemas_à_valider = "question_sys"
    elif "type" in contenu_fichier and contenu_fichier["type"].lower() == "bd":
        schemas_à_valider = "question_bd"
    else:
        schemas_à_valider = "question_base"

    return traiter_validation_question_yaml(contenu_fichier, schemas_à_valider)


def get_readers():
    class URLYamlReader(yamlinclude.YamlReader):
        def __call__(self):
            chemin = werkzeug.urls.iri_to_uri(self._path)
            if __args__.verbose:
                print(f"Inclusion du fichier YML : {chemin}...", file=sys.stderr)
            with urllib.request.urlopen(chemin) as fp:
                résultat = yaml.load(fp, self._loader_class)
            if __args__.verbose:
                print(f"Fin {chemin}", file=sys.stderr)
            return résultat

    class URLPlainTextReader(yamlinclude.PlainTextReader):
        def __call__(self):
            chemin = werkzeug.urls.iri_to_uri(self._path)
            if __args__.verbose:
                print(f"Inclusion du fichier texte : {chemin}...", file=sys.stderr)
            with urllib.request.urlopen(chemin) as fp:
                résultat = fp.read().decode("utf8")
            if __args__.verbose:
                print(f"Fin {chemin}", file=sys.stderr)
            return résultat

    return [
        (re.compile(r"^.+\.ya?ml$", re.IGNORECASE), URLYamlReader),  # *.yml, *.yaml
        (re.compile(r"^.+$"), URLPlainTextReader),  # *
    ]


def charger_question(cible, readers):
    url = urllib.parse.urlparse(cible)

    if url.scheme == "":
        base = pathlib.Path(os.path.abspath(os.path.dirname(cible))).as_uri()
    else:
        base = werkzeug.urls.iri_to_uri(os.path.dirname(cible))

    filename = os.path.basename(url.path)

    yamlinclude.YamlIncludeConstructor.add_to_loader_class(
        loader_class=yaml.SafeLoader, reader_map=readers, base_dir=base
    )

    if __args__.verbose:
        print(f"Chargement du fichier : {filename}")

    contenu_fichier = yaml.load(f"!include {filename}", Loader=yaml.SafeLoader)

    if not isinstance(contenu_fichier, dict):
        return None
    else:
        return contenu_fichier


def afficher_résultats_json(resultats):
    if "erreurs" in resultats and len(resultats["erreurs"]) > 0:
        afficher_erreurs_json(resultats["erreurs"])
        print("---", file=sys.stderr)

    if "avertissements" in resultats and len(resultats["avertissements"]) > 0:
        afficher_avertissements_json(resultats["avertissements"])
        print("---", file=sys.stderr)

    if "infos_question" in resultats:
        afficher_question_json(resultats["infos_question"])


def afficher_résultats(resultats):
    if "erreurs" in resultats and len(resultats["erreurs"]) > 0:
        afficher_erreurs(resultats["erreurs"])

    if "avertissements" in resultats and len(resultats["avertissements"]) > 0:
        afficher_avertissements(resultats["avertissements"])

    if "infos_question" in resultats:
        afficher_question(resultats["infos_question"])


def afficher_question(infos_question):
    print(yaml.dump(infos_question, allow_unicode=True))


def afficher_question_json(infos_question):
    print(json.dumps(infos_question, ensure_ascii=False))


def afficher_erreurs(erreurs):
    for erreur in erreurs:
        print(f"ERREUR : {erreur} > {erreurs[erreur]}", file=sys.stderr)


def afficher_erreurs_json(erreurs):
    print("{erreurs: " + str(erreurs) + "}", file=sys.stderr)


def afficher_avertissements(avertissements):
    for avertissement in avertissements:
        print(
            f"AVERTISSEMENT : {avertissement} > {avertissements[avertissement]}",
            file=sys.stderr,
        )


def afficher_avertissements_json(avertissements):
    print("{avertissements: " + str(avertissements) + "}", file=sys.stderr)


def traiter_paramètres():
    parser = argparse.ArgumentParser(
        description="progression_qc est un compilateur/validateur pour la production de d'exercices pour Progression. dprogression_qc reçoit sur l'entrée standard ou en paramètre un fichier YAML contenant la description d'une question et reproduit sur la sortie standard le résultat traité et validé."  # noqa: E501
    )
    verbosité = parser.add_mutually_exclusive_group()
    verbosité.add_argument(
        "-v", "--verbose", action="store_true", help="Affiche plus d'information."
    )
    verbosité.add_argument(
        "-q", "--quiet", action="store_true", help="Ne produit aucun affichage."
    )
    parser.add_argument(
        "-j",
        "--json",
        action="store_true",
        help="affiche le résultat et les erreurs/avertissements de validation en format JSON. Par défaut, le résultat est affiché en YAML et les erreurs/avertissements de la validation est en format humainement lisible.",  # noqa: E501
    )
    parser.add_argument(
        "--strict",
        action="store_true",
        help="l'exercice n'est considéré valide que si aucune erreur ni avertissement n'a été produit.",  # noqa: E501
    )
    parser.add_argument(
        "-V",
        "--version",
        action="store_true",
        help="Affiche la version de progression_qc et termine.",
    )
    parser.add_argument(
        "fichier",
        type=str,
        nargs="?",
        help="Le fichier YML à valider.",
        default="/dev/stdin",
    )

    global __args__
    __args__ = parser.parse_args()

    return __args__


def déterminer_code_retour(resultats):
    if "erreurs" in resultats and len(resultats["erreurs"]) > 0:
        return 1
    elif "avertissements" in resultats and len(resultats["avertissements"]) > 0:
        return 2 if __args__.strict else 0
    return 0



"""
progression_qc est un compilateur/validateur pour la production de d'exercices pour Progression. progression_qc reçoit sur l'entrée standard ou en paramètre un fichier YAML contenant la description d'une question et reproduit sur la sortie standard le résultat traité et validé.
"""

import os

print(os.environ)

__version__ = os.environ["VERSION"]
__author__ = "Patrick Lafrance"



from progression_qc import progression_qc

def test_valider_question_sys_complète_valide_avec_tests():
    from question_sys_complète_valide_avec_tests import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs":{}}

def test_valider_question_sys_complète_valide_avec_réponse():
    from question_sys_complète_valide_avec_réponse import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs":{}}

def test_valider_question_sys_minimale_valide_avec_tests():
    from question_sys_minimale_valide_avec_tests import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs":{}}

def test_valider_question_sys_minimale_valide_avec_réponse():
    from question_sys_minimale_valide_avec_réponse import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs":{}}

def test_valider_question_sys_sans_image_ni_tests():
    from question_sys_sans_image_ni_tests import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{},
                        'erreurs': {'tests': ['required field'],
                                    'image': ['required field'],
                                    'réponse': ['required field']}}

def test_valider_question_sys_test_sans_sortie():
    from question_sys_test_sans_sortie import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {"tests": [{0: [{"sortie": ["required field"]}]}]}}


def test_valider_question_sys_avec_image_et_tests_vides():
    from question_sys_avec_image_et_tests_vides import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{},
                        'erreurs': {'tests': ['empty values not allowed'],
                                    'image': ['empty values not allowed']}}

def test_valider_question_sys_avec_image_et_réponse_vides():
    from question_sys_avec_image_et_réponse_vides import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{},
                        'erreurs': {'réponse': ['empty values not allowed'],
                                    'image': ['empty values not allowed']}}

def test_valider_question_sys_test_sortie_nulle():
    from question_sys_test_sortie_nulle import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {"tests": [{0: [{"sortie": ["null value not allowed"]}]}]}}

def test_valider_question_sys_utilisateur_nul():
    from question_sys_utilisateur_nul import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {"utilisateur": ["null value not allowed"]}}

def test_valider_question_sys_utilisateur_invalide():
    from question_sys_utilisateur_invalide import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {"utilisateur": ["value does not match regex '^[a-z][-a-z0-9_]*$'"]}}

def test_valider_question_sys_init_nul():
    from question_sys_init_nul import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {"init": ["null value not allowed"]}}

def test_valider_question_sys_init_vide():
    from question_sys_init_vide import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {"init": ["empty values not allowed"]}}

def test_valider_question_prog_complète():
    from question_prog_complète_valide import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {}}

def test_valider_question_avec_dict_pour_énoncés():
    from question_avec_dict_pour_énoncés import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {}}

def test_valider_question_prog_minimale():
    from question_prog_minimale_valide import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {}}

def test_valider_question_avec_avertissement():
    from question_avec_avertissement import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {'tata': ['unknown field']}, "erreurs": {}}

def test_valider_question_avec_erreur():
    from question_avec_erreur import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {'type': ['required field']}}

def test_valider_question_prog_test_sans_sortie():
    from question_prog_test_sans_sortie import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {"tests": [{0: [{"sortie": ["required field"]}]}]}}

def test_valider_question_prog_test_avec_sortie_nulle():
    from question_prog_test_sortie_nulle import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {"tests": [{0: [{"sortie": ["null value not allowed"]}]}]}}

def test_valider_question_prog_test_avec_params_nul():
    from question_prog_test_params_nul import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {"tests": [{0: [{"params": ["null value not allowed"]}]}]}}

def test_valider_question_prog_sans_ébauches_ni_tests():
    from question_prog_sans_ébauches_ni_tests import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {},
                        'erreurs': {'tests': ['required field'],
                                    'ébauches': ['required field']}}

def test_valider_question_prog_avec_ébauches_et_tests_vides():
    from question_prog_avec_ébauches_et_tests_vides import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {},
                        'erreurs': {'tests': ['empty values not allowed'],
                                    'ébauches': ['empty values not allowed']}}

def test_valider_question_vcs_minimale():
    from question_vcs_minimale import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {}}

def test_valider_question_vsc_sans_champ_vcs():
    from question_vcs_sans_champ_vcs import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {'vcs': ['required field']}}

def test_valider_question_vsc_avec_champ_vcs_null():
    from question_vcs_avec_champ_vcs_null import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements": {}, "erreurs": {'vcs': ['null value not allowed']}}
    
def test_valider_question_vsc_avec_champ_vcs_vide():
    from question_vcs_avec_champ_vcs_vide import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)
    print (résultat)
    assert résultat == {'erreurs': {'vcs': ['empty values not allowed']}, 'avertissements': {}}

def test_valider_question_bd_sans_sgbd():
    from question_bd_sans_sgbd import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{},
                        'erreurs': {'sgbd': ['required field']}}

def test_valider_question_bd_avec_sgbd_vide():
    from question_bd_avec_sgbd_vide import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {'sgbd': ['empty values not allowed']}}

def test_valider_question_bd_avec_sgbd_valide():
    from question_bd_avec_sgbd_valide import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs":{}}

def test_valider_question_bd_avec_sgbd_null():
    from question_bd_avec_sgbd_null import question

    résultat = progression_qc.valider_schema_yaml_infos_question(question)

    assert résultat == {"avertissements":{}, "erreurs": {"sgbd": ["null value not allowed"]}}
